@@ -1,5 +1,5 @@
 /* html.c -- html-related utilities.
-   $Id: html.c,v 1.18 2004/03/20 22:17:10 karl Exp $
+   $Id: html.c,v 1.27 2004/11/30 02:03:23 karl Exp $
 
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Free Software
    Foundation, Inc.
@@ -308,12 +308,16 @@ rel=\"generator-home\" title=\"Texinfo Homepage\">\n");
     /* Since HTML doesn't have a sc element, we use span with a bit of
        CSS spice instead.  */
 #define CSS_FONT_SMALL_CAPS "font-variant:small-caps"
-    add_word_args ("  span.sc { %s }\n", CSS_FONT_SMALL_CAPS);
+    add_word_args ("  span.sc    { %s }\n", CSS_FONT_SMALL_CAPS);
 
-    /* Roman font class.  */
-#define CSS_FONT_ROMAN "font-family: serif; font-weight: normal;"
+    /* Roman (default) font class, closest we can come.  */
+#define CSS_FONT_ROMAN "font-family:serif; font-weight:normal;"
     add_word_args ("  span.roman { %s } \n", CSS_FONT_ROMAN);
 
+    /* Sans serif font class.  */
+#define CSS_FONT_SANSSERIF "font-family:sans-serif; font-weight:normal;"
+    add_word_args ("  span.sansserif { %s } \n", CSS_FONT_SANSSERIF);
+
     /* Write out any css code from the user's --css-file.  */
     if (css_inline)
       insert_string (css_inline);
@@ -587,9 +591,17 @@ add_link (char *nodename, char *attributes)
 }
 
 /* Output NAME with characters escaped as appropriate for an anchor
-   name, i.e., escape URL special characters as %<n>.  */
+   name, i.e., escape URL special characters with our _00hh convention
+   if OLD is zero.  (See the manual for details on the new scheme.)
+   
+   If OLD is nonzero, generate the node name with the 4.6-and-earlier
+   convention of %hh (and more special characters output as-is, notably
+   - and *).  This is only so that external references to old names can
+   still work with HTML generated by the new makeinfo; the gcc folks
+   needed this.  Our own HTML does not refer to these names.  */
+
 void
-add_escaped_anchor_name (char *name)
+add_escaped_anchor_name (char *name, int old)
 {
   canon_white (name);
 
@@ -597,23 +609,33 @@ add_escaped_anchor_name (char *name)
     {
       if (cr_or_whitespace (*name))
         add_char ('-');
-      else if (! URL_SAFE_CHAR (*name))
+
+      else if (!old && !URL_SAFE_CHAR (*name))
         /* Cast so characters with the high bit set are treated as >128,
            for example o-umlaut should be 246, not -10.  */
         add_word_args ("_00%x", (unsigned char) *name);
+
+      else if (old && !URL_SAFE_CHAR (*name) && !OLD_URL_SAFE_CHAR (*name))
+        /* Different output convention, but still cast as above.  */
+        add_word_args ("%%%x", (unsigned char) *name);
+
       else
         add_char (*name);
     }
 }
 
 /* Insert the text for the name of a reference in an HTML anchor
-   appropriate for NODENAME.  If HREF is nonzero, it will be
-   appropriate for a href= attribute, rather than name= i.e., including
-   the `#' if it's an internal reference. */
+   appropriate for NODENAME.
+   
+   If HREF is zero, generate text for name= in the new node name
+     conversion convention.
+   If HREF is negative, generate text for name= in the old convention.
+   If HREF is positive, generate the name for an href= attribute, i.e.,
+     including the `#' if it's an internal reference.   */
 void
 add_anchor_name (char *nodename, int href)
 {
-  if (href)
+  if (href > 0)
     {
       if (splitting)
 	add_url_name (nodename, href);
@@ -629,7 +651,7 @@ add_anchor_name (char *nodename, int href)
   else if (strcasecmp (nodename, "top") == 0)
     add_word ("Top");
   else
-    add_escaped_anchor_name (nodename);
+    add_escaped_anchor_name (nodename, href < 0);
 }
 
 /* Insert the text for the name of a reference in an HTML url, aprropriate
